﻿using System;
using System.Diagnostics;
using System.IO;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using Microsoft.Win32; // Update to use OpenFileDialog from Microsoft.Win32
using MessageBox = System.Windows.MessageBox; // Alias to avoid conflict with WinForms
using Microsoft.WindowsAPICodePack.Dialogs;

namespace ACSuite.Views
{
    public partial class VideoConverter : Window
    {
        private CancellationTokenSource _cancellationTokenSource;

        public VideoConverter()
        {
            InitializeComponent();
            this.Loaded += VideoConverter_Loaded;

            // Add event handlers for TextBox focus changes
            InputDirectoryTextBox.LostFocus += InputDirectoryTextBox_LostFocus;
            OutputDirectoryTextBox.LostFocus += OutputDirectoryTextBox_LostFocus;
        }

        private void VideoConverter_Loaded(object sender, RoutedEventArgs e)
        {
            if (FileFormatComboBox.SelectedItem != null)
            {
                FileFormatComboBox_SelectionChanged(this, null); // Initialize quality slider based on default format
            }
        }

        private void BrowseInputDirectory_Click(object sender, RoutedEventArgs e)
        {
            var dialog = new CommonOpenFileDialog
            {
                IsFolderPicker = true
            };

            if (dialog.ShowDialog() == CommonFileDialogResult.Ok)
            {
                InputDirectoryTextBox.Text = dialog.FileName;
                UpdateValidVideosCount();
            }
        }

        private void BrowseOutputDirectory_Click(object sender, RoutedEventArgs e)
        {
            var dialog = new CommonOpenFileDialog
            {
                IsFolderPicker = true
            };

            if (dialog.ShowDialog() == CommonFileDialogResult.Ok)
            {
                OutputDirectoryTextBox.Text = dialog.FileName;
            }
        }

        private void InputDirectoryTextBox_LostFocus(object sender, RoutedEventArgs e)
        {
            UpdateValidVideosCount();
        }

        private void OutputDirectoryTextBox_LostFocus(object sender, RoutedEventArgs e)
        {
            // Logic to handle focus lost on OutputDirectoryTextBox if needed
        }

        private void FileFormatComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (FileFormatComboBox.SelectedItem == null)
                return;

            string selectedFormat = (FileFormatComboBox.SelectedItem as ComboBoxItem)?.Content.ToString();
            if (selectedFormat == null)
                return;

            if (selectedFormat == "webm")
            {
                QualitySlider.Minimum = 0;
                QualitySlider.Maximum = 63;
            }
            else if (selectedFormat == "mp4")
            {
                QualitySlider.Minimum = 18;
                QualitySlider.Maximum = 51;
            }
            // Add other formats and their quality ranges here

            QualitySlider.Value = QualitySlider.Maximum;
            QualityValueTextBlock.Text = $"Quality: {(int)QualitySlider.Value}";
        }

        private void QualitySlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (QualityValueTextBlock != null)
            {
                QualityValueTextBlock.Text = $"Quality: {(int)QualitySlider.Value}";
            }
        }

        private void ResizeSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (ResizeValueTextBlock != null)
            {
                ResizeValueTextBlock.Text = $"Resize: {(int)ResizeSlider.Value}%";
            }
        }

        private void UpdateValidVideosCount()
        {
            try
            {
                string inputDir = InputDirectoryTextBox.Text;
                string fileFormat = (FileFormatComboBox.SelectedItem as ComboBoxItem)?.Content.ToString();
                if (fileFormat == null)
                    return;

                int validVideosCount = 0;

                foreach (string filePath in Directory.GetFiles(inputDir, $"*.{fileFormat}"))
                {
                    string ffprobeArgs = $"-v error -select_streams v:0 -show_entries stream=codec_type -of default=noprint_wrappers=1:nokey=1 \"{filePath}\"";
                    string ffprobeOutput = ExecuteCommand("ffprobe", ffprobeArgs);

                    if (ffprobeOutput.Trim() == "video")
                    {
                        validVideosCount++;
                    }
                }

                ValidVideosCountTextBlock.Text = $"Valid videos found: {validVideosCount}";
            }
            catch (Exception ex)
            {
                MessageBox.Show($"Error: {ex.Message}", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private async void ConvertButton_Click(object sender, RoutedEventArgs e)
        {
            string inputDir = InputDirectoryTextBox.Text;
            string outputDir = OutputDirectoryTextBox.Text;
            string quality = QualitySlider.Value.ToString();
            string fileFormat = (FileFormatComboBox.SelectedItem as ComboBoxItem)?.Content.ToString();
            int resizePercentage = (int)ResizeSlider.Value;

            if (fileFormat == null)
                return;

            try
            {
                _cancellationTokenSource = new CancellationTokenSource();
                StopButton.IsEnabled = true;
                ConvertButton.IsEnabled = false;

                ConversionProgressBar.Visibility = Visibility.Visible;
                ProgressTextBlock.Visibility = Visibility.Visible;

                await Task.Run(() => ConvertVideo(inputDir, outputDir, quality, fileFormat, resizePercentage, _cancellationTokenSource.Token));

                MessageBox.Show("Conversion completed.", "Conversion Complete", MessageBoxButton.OK, MessageBoxImage.Information);
            }
            catch (OperationCanceledException)
            {
                MessageBox.Show("Conversion canceled.", "Canceled", MessageBoxButton.OK, MessageBoxImage.Information);
            }
            catch (Exception ex)
            {
                MessageBox.Show($"Error: {ex.Message}", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
            finally
            {
                ConversionProgressBar.Visibility = Visibility.Collapsed;
                ProgressTextBlock.Visibility = Visibility.Collapsed;
                StopButton.IsEnabled = false;
                ConvertButton.IsEnabled = true;
            }
        }

        private void ConvertVideo(string inputDir, string outputDir, string quality, string fileFormat, int resizePercentage, CancellationToken token)
        {
            try
            {
                Directory.CreateDirectory(outputDir);

                string[] files = Directory.GetFiles(inputDir, $"*.{fileFormat}");
                int totalFiles = files.Length;
                int processedFiles = 0;

                foreach (string filePath in files)
                {
                    token.ThrowIfCancellationRequested();

                    string fileName = Path.GetFileName(filePath);

                    string ffprobeArgs = $"-v error -select_streams v:0 -show_entries stream=width,height -of csv=p=0 \"{filePath}\"";
                    string ffprobeOutput = ExecuteCommand("ffprobe", ffprobeArgs);

                    var dimensions = ffprobeOutput.Trim().Split(',');
                    if (dimensions.Length < 2) continue;

                    int originalWidth = int.Parse(dimensions[0]);
                    int originalHeight = int.Parse(dimensions[1]);

                    int newWidth = originalWidth * resizePercentage / 100;
                    int newHeight = originalHeight * resizePercentage / 100;

                    string outputPath = Path.Combine(outputDir, Path.GetFileNameWithoutExtension(fileName) + ".webm");

                    // Check if the file already exists
                    if (File.Exists(outputPath))
                    {
                        Dispatcher.Invoke(() =>
                        {
                            var result = MessageBox.Show($"The file '{outputPath}' already exists. Do you want to overwrite it?", "File Exists", MessageBoxButton.YesNoCancel, MessageBoxImage.Question);

                            switch (result)
                            {
                                case MessageBoxResult.Yes:
                                    // Overwrite the file
                                    break;
                                case MessageBoxResult.No:
                                    // Rename the file
                                    outputPath = Path.Combine(outputDir, Path.GetFileNameWithoutExtension(fileName) + "1.webm");
                                    break;
                                case MessageBoxResult.Cancel:
                                    // Cancel the conversion for this file
                                    return;
                            }
                        });
                    }

                    string ffmpegArgs = $"-i \"{filePath}\" -vf scale={newWidth}:{newHeight} -c:v libvpx -crf {quality} -b:v 1M -c:a libvorbis \"{outputPath}\"";
                    ExecuteCommand("ffmpeg", ffmpegArgs, token);

                    processedFiles++;
                    Dispatcher.Invoke(() =>
                    {
                        ConversionProgressBar.Value = (double)processedFiles / totalFiles * 100;
                        ProgressTextBlock.Text = $"Progress: {(double)processedFiles / totalFiles * 100:0.0}%";
                    });
                }
            }
            catch (Exception ex)
            {
                throw new Exception($"Error during conversion: {ex.Message}");
            }
        }

        private string ExecuteCommand(string cmd, string args, CancellationToken token = default)
        {
            ProcessStartInfo startInfo = new ProcessStartInfo
            {
                FileName = cmd,
                Arguments = args,
                RedirectStandardOutput = true,
                UseShellExecute = false,
                CreateNoWindow = true
            };

            using (Process process = Process.Start(startInfo))
            {
                using (StreamReader reader = process.StandardOutput)
                {
                    while (!reader.EndOfStream)
                    {
                        token.ThrowIfCancellationRequested();
                        return reader.ReadToEnd();
                    }
                }
            }

            return string.Empty;
        }

        private void StopButton_Click(object sender, RoutedEventArgs e)
        {
            _cancellationTokenSource?.Cancel();
        }
    }
}
