﻿using ACSuite.Views;
using Microsoft.Win32;
using SixLabors.ImageSharp;
using SixLabors.ImageSharp.Formats;
using SixLabors.ImageSharp.Formats.Webp;
using SixLabors.ImageSharp.PixelFormats;
using SixLabors.ImageSharp.Processing;
using System;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Forms;
using System.Threading.Tasks;

namespace ACSuite.Views
{
    public partial class ImageConverter : Window
    {
        public ImageConverter()
        {
            InitializeComponent();
            Loaded += ImageConverter_Loaded;
        }

        private void ImageConverter_Loaded(object sender, RoutedEventArgs e)
        {
            BatchConversionRadio.IsChecked = true;
            NoResizeRadio.IsChecked = true;
        }

        private void ConvertButton_Click(object sender, RoutedEventArgs e)
        {
            string conversionType = BatchConversionRadio.IsChecked == true ? "1" : "2";
            string resizeOption = NoResizeRadio.IsChecked == true ? "0" : PercentageResizeRadio.IsChecked == true ? "1" : "2";
            string resizeValueText = resizeOption == "1" ? ResizePercentageTextBox.Text : $"{ResizeWidthTextBox.Text},{ResizeHeightTextBox.Text}";
            string outputFormat = ((ComboBoxItem)OutputFormatComboBox.SelectedItem)?.Content.ToString().ToLower();
            string inputPath = InputPathTextBox.Text;
            string outputPath = OutputPathTextBox.Text;

            if (string.IsNullOrEmpty(outputFormat) || string.IsNullOrEmpty(inputPath) || string.IsNullOrEmpty(outputPath))
            {
                StatusTextBlock.Text = "Please fill in all required fields.";
                return;
            }

            if (conversionType == "1" && resizeOption == "2")
            {
                var result = System.Windows.MessageBox.Show("Batch Conversion with Resize by Pixels will apply the same absolute dimensions to all images, which may look weird for mixed orientations. Do you want to proceed?", "Warning", MessageBoxButton.YesNo, MessageBoxImage.Warning);
                if (result == MessageBoxResult.No)
                {
                    return;
                }
            }

            if (conversionType == "1")
            {
                ConvertBatch(inputPath, outputPath, resizeOption, resizeValueText, outputFormat);
            }
            else
            {
                ConvertSingleFile(inputPath, outputPath, outputFormat, resizeOption, resizeValueText);
            }
        }

        private void ConvertSingleFile(string inputFile, string outputFolder, string outputFormat, string resizeOption, string resizeValueText)
        {
            var resizeValue = GetResizeValue(resizeOption, resizeValueText, out bool useOpenCV);
            string outputFile = Path.Combine(outputFolder, Path.GetFileNameWithoutExtension(inputFile) + "." + outputFormat);
            ConvertImage(inputFile, outputFile, outputFormat, resizeOption, resizeValue, useOpenCV);
        }

        private async void ConvertBatch(string inputFolder, string outputFolder, string resizeOption, string resizeValueText, string outputFormat)
        {
            if (!Directory.Exists(outputFolder))
            {
                Directory.CreateDirectory(outputFolder);
            }

            var files = Directory.GetFiles(inputFolder);
            var validFiles = files.Where(file => IsValidImage(file)).ToList();
            int totalFiles = validFiles.Count;
            int convertedFiles = 0;

            var resizeValue = GetResizeValue(resizeOption, resizeValueText, out bool useOpenCV);

            foreach (var inputFile in validFiles)
            {
                string outputFile = Path.Combine(outputFolder, Path.GetFileNameWithoutExtension(inputFile) + "." + outputFormat);
                if (ConvertImage(inputFile, outputFile, outputFormat, resizeOption, resizeValue, useOpenCV))
                {
                    convertedFiles++;
                    StatusTextBlock.Text = $"Converted {convertedFiles}/{totalFiles} files.";
                    await Task.Delay(50); // Simulate work
                }
            }

            StatusTextBlock.Text = $"Conversion completed. {convertedFiles}/{totalFiles} files converted.";
        }

        private (int, int) GetResizeValue(string resizeOption, string resizeValueText, out bool useOpenCV)
        {
            useOpenCV = false;
            if (resizeOption == "1")
            {
                if (double.TryParse(resizeValueText.TrimEnd('%'), out double percentage))
                {
                    return ((int)percentage, 0);
                }
            }
            else if (resizeOption == "2")
            {
                var dimensions = resizeValueText.Split(',');
                if (dimensions.Length == 2 && int.TryParse(dimensions[0], out int width) && int.TryParse(dimensions[1], out int height))
                {
                    return (width, height);
                }
            }

            return (0, 0);
        }

        private bool ConvertImage(string inputFile, string outputFile, string outputFormat, string resizeOption, (int, int) resizeValue, bool useOpenCV)
        {
            try
            {
                using (var image = SixLabors.ImageSharp.Image.Load(inputFile))
                {
                    if (resizeOption == "1" && resizeValue.Item1 > 0)
                    {
                        int newWidth = (int)(image.Width * resizeValue.Item1 / 100.0);
                        int newHeight = (int)(image.Height * resizeValue.Item1 / 100.0);
                        image.Mutate(x => x.Resize(newWidth, newHeight));
                    }
                    else if (resizeOption == "2" && resizeValue.Item1 > 0 && resizeValue.Item2 > 0)
                    {
                        image.Mutate(x => x.Resize(resizeValue.Item1, resizeValue.Item2));
                    }

                    IImageEncoder encoder = GetImageEncoder(outputFormat);
                    image.Save(outputFile, encoder);
                }
                return true;
            }
            catch (Exception ex)
            {
                StatusTextBlock.Text = $"Error: {ex.Message}";
                return false;
            }
        }

        private IImageEncoder GetImageEncoder(string format)
        {
            return format switch
            {
                "png" => new SixLabors.ImageSharp.Formats.Png.PngEncoder(),
                "jpg" => new SixLabors.ImageSharp.Formats.Jpeg.JpegEncoder(),
                "jpeg" => new SixLabors.ImageSharp.Formats.Jpeg.JpegEncoder(),
                "webp" => new SixLabors.ImageSharp.Formats.Webp.WebpEncoder(),
                "gif" => new SixLabors.ImageSharp.Formats.Gif.GifEncoder(),
                "tiff" => new SixLabors.ImageSharp.Formats.Tiff.TiffEncoder(),
                "bmp" => new SixLabors.ImageSharp.Formats.Bmp.BmpEncoder(),
                _ => new SixLabors.ImageSharp.Formats.Png.PngEncoder(),
            };
        }

        private void BrowseInputPath_Click(object sender, RoutedEventArgs e)
        {
            if (BatchConversionRadio.IsChecked == true)
            {
                using var dialog = new FolderBrowserDialog();
                if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    InputPathTextBox.Text = dialog.SelectedPath;
                    UpdateValidImageCount(dialog.SelectedPath);
                }
            }
            else
            {
                var dialog = new Microsoft.Win32.OpenFileDialog();
                dialog.Filter = "Image Files|*.jpg;*.jpeg;*.png;*.bmp;*.gif;*.tiff;*.webp";
                if (dialog.ShowDialog() == true)
                {
                    InputPathTextBox.Text = dialog.FileName;
                    UpdateValidImageCount(Path.GetDirectoryName(dialog.FileName));
                }
            }
        }

        private void BrowseOutputPath_Click(object sender, RoutedEventArgs e)
        {
            using var dialog = new FolderBrowserDialog();
            if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                OutputPathTextBox.Text = dialog.SelectedPath;
            }
        }

        private void UpdateValidImageCount(string path)
        {
            if (BatchConversionRadio.IsChecked == true)
            {
                var files = Directory.GetFiles(path);
                var validFiles = files.Count(file => IsValidImage(file));
                StatusTextBlock.Text = $"Found {validFiles} valid images in the selected directory.";
            }
            else
            {
                StatusTextBlock.Text = IsValidImage(InputPathTextBox.Text) ? "1 valid image selected." : "No valid image selected.";
            }
        }

        private bool IsValidImage(string filePath)
        {
            try
            {
                return SixLabors.ImageSharp.Image.Identify(filePath) != null;
            }
            catch
            {
                return false;
            }
        }

        private void TextBox_GotFocus(object sender, RoutedEventArgs e)
        {
            if (sender is System.Windows.Controls.TextBox textBox)
            {
                textBox.Text = string.Empty;
            }
        }

        private void ResizePercentageTextBox_LostFocus(object sender, RoutedEventArgs e)
        {
            if (sender is System.Windows.Controls.TextBox textBox)
            {
                if (double.TryParse(textBox.Text.TrimEnd('%'), out _))
                {
                    // Valid percentage, do nothing
                }
                else
                {
                    textBox.Text = "100";
                }
            }
        }

        private void ResizeWidthTextBox_LostFocus(object sender, RoutedEventArgs e)
        {
            if (sender is System.Windows.Controls.TextBox textBox)
            {
                if (int.TryParse(textBox.Text, out _))
                {
                    // Valid width, do nothing
                }
                else
                {
                    textBox.Text = "Width";
                }
            }
        }

        private void ResizeHeightTextBox_LostFocus(object sender, RoutedEventArgs e)
        {
            if (sender is System.Windows.Controls.TextBox textBox)
            {
                if (int.TryParse(textBox.Text, out _))
                {
                    // Valid height, do nothing
                }
                else
                {
                    textBox.Text = "Height";
                }
            }
        }
    }
}
