﻿using System;
using System.Windows;
using System.Windows.Controls;

namespace ACSuite.Views
{
    public partial class LeadCalculator : Window
    {
        private double weightedSumA = 0;
        private double weightedSumB = 0;
        private double totalSampleSize = 0;

        public LeadCalculator()
        {
            InitializeComponent();
        }

        private void PollTabControl_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (PollTabControl.SelectedIndex == PollTabControl.Items.Count - 1)
            {
                AddPollTab();
            }
        }

        private void AddPollTab()
        {
            var newTab = new TabItem
            {
                Header = $"Poll {PollTabControl.Items.Count - 1}",
                Content = CreatePollContent()
            };

            PollTabControl.Items.Insert(PollTabControl.Items.Count - 1, newTab);
            PollTabControl.SelectedItem = newTab;
        }

        private StackPanel CreatePollContent()
        {
            var stackPanel = new StackPanel { Margin = new Thickness(10) };

            stackPanel.Children.Add(new TextBlock { Text = "Enter polling % for Candidate A:" });
            var candidateATextBox = new TextBox { Width = 200 };
            stackPanel.Children.Add(candidateATextBox);

            stackPanel.Children.Add(new TextBlock { Text = "Enter polling % for Candidate B:", Margin = new Thickness(0, 10, 0, 0) });
            var candidateBTextBox = new TextBox { Width = 200 };
            stackPanel.Children.Add(candidateBTextBox);

            stackPanel.Children.Add(new TextBlock { Text = "Enter sample size:", Margin = new Thickness(0, 10, 0, 0) });
            var sampleSizeTextBox = new TextBox { Width = 200 };
            stackPanel.Children.Add(sampleSizeTextBox);

            return stackPanel;
        }

        private void CalculateButton_Click(object sender, RoutedEventArgs e)
        {
            weightedSumA = 0;
            weightedSumB = 0;
            totalSampleSize = 0;

            // Skip the first tab (Results tab) and the last tab (Add Poll tab)
            for (int i = 1; i < PollTabControl.Items.Count - 1; i++)
            {
                if (PollTabControl.Items[i] is TabItem tabItem && tabItem.Content is StackPanel stackPanel)
                {
                    var candidateATextBox = stackPanel.Children[1] as TextBox;
                    var candidateBTextBox = stackPanel.Children[3] as TextBox;
                    var sampleSizeTextBox = stackPanel.Children[5] as TextBox;

                    if (candidateATextBox != null && candidateBTextBox != null && sampleSizeTextBox != null &&
                        double.TryParse(candidateATextBox.Text, out double a) &&
                        double.TryParse(candidateBTextBox.Text, out double b) &&
                        double.TryParse(sampleSizeTextBox.Text, out double c) &&
                        a > 0 && b > 0 && c > 0)
                    {
                        weightedSumA += a * c;
                        weightedSumB += b * c;
                        totalSampleSize += c;
                    }
                }
            }

            if (totalSampleSize > 0)
            {
                double weightedAvgA = weightedSumA / totalSampleSize;
                double weightedAvgB = weightedSumB / totalSampleSize;

                var (prob, upperBound, lowerBound) = DoMath(weightedAvgA, weightedAvgB, totalSampleSize);
                ResultTextBlock.Text = $"Probability of Candidate A actually being in the lead: {prob}%";
            }
            else
            {
                MessageBox.Show("Please add at least one poll.");
            }
        }

        private void ResetButton_Click(object sender, RoutedEventArgs e)
        {
            PollTabControl.Items.Clear();

            var resultsTab = new TabItem
            {
                Header = "Results",
                Content = new StackPanel
                {
                    Margin = new Thickness(10),
                    Children =
                    {
                        new Button { Name = "CalculateButton", Content = "Calculate Probability", Width = 200, Margin = new Thickness(0,0,0,10) },
                        new Button { Name = "ResetButton", Content = "Reset All Polls", Width = 200, Margin = new Thickness(0,0,0,10) },
                        new TextBlock { Name = "ResultTextBlock", Margin = new Thickness(20,10,0,0) }
                    }
                }
            };
            ((Button)((StackPanel)resultsTab.Content).Children[0]).Click += CalculateButton_Click;
            ((Button)((StackPanel)resultsTab.Content).Children[1]).Click += ResetButton_Click;

            var addPollTab = new TabItem
            {
                Header = "Add Poll",
                Content = new TextBlock
                {
                    Text = "Click here to add a new poll",
                    HorizontalAlignment = HorizontalAlignment.Center,
                    VerticalAlignment = VerticalAlignment.Center
                }
            };

            PollTabControl.Items.Add(resultsTab);
            PollTabControl.Items.Add(addPollTab);
        }

        private (double, double, double) DoMath(double a, double b, double c)
        {
            double c1 = (((a * 0.01) + (b * 0.01)) * c);
            double d = Math.Sqrt((((a * 0.01) + (b * 0.01)) - (Math.Pow(((a * 0.01) - (b * 0.01)), 2))) / c1);
            double upperBound = ((Math.Abs(a - b)) + (d * 1.96) * 100) + (0.5 / c1);
            double lowerBound = ((Math.Abs(a - b)) - (d * 1.96) * 100) - (0.5 / c1);
            double prob = Normald(((a - b) * 0.01) / d);
            return (Math.Round(prob * 100, 2), Math.Round(upperBound, 1), Math.Round(lowerBound, 1));
        }

        private double Normald(double x)
        {
            double t = 1 / (1 + 0.2316419 * Math.Abs(x));
            double m = 0.398942280401 * Math.Exp(-x * x / 2);
            double prob = m * t * (0.319381530 + t * (-0.356563782 + t * (1.781477937 + t * (-1.821255978 + t * 1.330274429))));
            if (x > 0)
            {
                prob = 1 - prob;
            }
            return prob;
        }
    }
}
