﻿using Microsoft.Win32;
using System;
using System.Diagnostics;
using System.IO;
using System.Text.RegularExpressions;
using System.Windows;

namespace ACSuite.Views
{
    public partial class VideoTrimmer : Window
    {
        public VideoTrimmer()
        {
            InitializeComponent();
        }

        private void BrowseButton_Click(object sender, RoutedEventArgs e)
        {
            OpenFileDialog openFileDialog = new OpenFileDialog
            {
                Filter = "Video Files (*.mp4;*.webm)|*.mp4;*.webm|All Files (*.*)|*.*"
            };
            if (openFileDialog.ShowDialog() == true)
            {
                InputFileNameTextBox.Text = openFileDialog.FileName;
            }
        }

        private void TrimVideoButton_Click(object sender, RoutedEventArgs e)
        {
            string inputFile = InputFileNameTextBox.Text;
            string startTimeInput = StartTimeTextBox.Text;
            string endTimeInput = EndTimeTextBox.Text;

            if (!File.Exists(inputFile))
            {
                OutputTextBlock.Text = "Video file not found. Supported formats: mp4, webm";
                return;
            }

            int startTime = ParseTimeInput(startTimeInput);
            int? endTime = !string.IsNullOrEmpty(endTimeInput) ? (int?)ParseTimeInput(endTimeInput) : null;

            // Determine the output filename based on the input file format
            string directory = Path.GetDirectoryName(inputFile);
            string filename = Path.GetFileNameWithoutExtension(inputFile);
            string extension = Path.GetExtension(inputFile);
            string outputFile = Path.Combine(directory, $"{filename}_trimmed{extension}");

            TrimVideo(inputFile, outputFile, startTime, endTime);
        }

        private int ParseTimeInput(string timeInput)
        {
            // Regular expression to parse time input like "1h3m39s" or "8s"
            var timePattern = @"(?:(\d+)h)?(?:(\d+)m)?(?:(\d+)s)?";
            var match = Regex.Match(timeInput, timePattern);

            int hours = match.Groups[1].Success ? int.Parse(match.Groups[1].Value) : 0;
            int minutes = match.Groups[2].Success ? int.Parse(match.Groups[2].Value) : 0;
            int seconds = match.Groups[3].Success ? int.Parse(match.Groups[3].Value) : 0;

            return hours * 3600 + minutes * 60 + seconds;
        }

        private string FormatTimeInput(int timeInSeconds)
        {
            TimeSpan timeSpan = TimeSpan.FromSeconds(timeInSeconds);
            return timeSpan.ToString(@"hh\:mm\:ss");
        }

        private async void TrimVideo(string inputFile, string outputFile, int startTime, int? endTime)
        {
            var startTimeFormatted = FormatTimeInput(startTime);
            var endTimeFormatted = endTime.HasValue ? FormatTimeInput(endTime.Value) : null;

            var arguments = $"-i \"{inputFile}\" -ss {startTimeFormatted}" +
                            (endTimeFormatted != null ? $" -to {endTimeFormatted}" : "") +
                            $" \"{outputFile}\"";

            OutputTextBlock.Text = $"Running command: ffmpeg {arguments}"; // For debugging

            var cmd = new ProcessStartInfo
            {
                FileName = "ffmpeg",
                Arguments = arguments,
                RedirectStandardOutput = true,
                RedirectStandardError = true,
                UseShellExecute = false,
                CreateNoWindow = true
            };

            try
            {
                using (var process = new Process())
                {
                    process.StartInfo = cmd;
                    process.OutputDataReceived += (sender, e) => { /* You can log output data here if needed */ };
                    process.ErrorDataReceived += (sender, e) => { /* You can log error data here if needed */ };
                    process.Start();

                    process.BeginOutputReadLine();
                    process.BeginErrorReadLine();

                    await process.WaitForExitAsync();

                    if (process.ExitCode == 0)
                    {
                        OutputTextBlock.Text = "Video trimmed successfully!";
                    }
                    else
                    {
                        OutputTextBlock.Text = $"Error trimming video: {await process.StandardError.ReadToEndAsync()}";
                    }
                }
            }
            catch (Exception ex)
            {
                OutputTextBlock.Text = $"An error occurred: {ex.Message}";
            }
        }
    }
}
